@@ -7,9 +7,10 @@ import json
 import multiprocessing
 import pathlib
 import shutil
+import sys
 import tempfile
 import uuid
-from io import BytesIO
+from io import BytesIO, StringIO
 from pathlib import Path
 from typing import Dict, List
 
@@ -43,11 +44,86 @@ ctx_forkserver.set_forkserver_preload(
 
 EXAMPLES = Path(__file__).parent / "examples"
 
+ALERT_STYLE = """
+<style>
+.bk.alert {
+    padding: 0.75rem 1.25rem;
+    border: 1px solid transparent;
+    border-radius: 0.25rem;
+    margin-top: 15px;
+    margin-bottom: 15px;
+}
+.bk.alert a {
+    color: rgb(11, 46, 19); /* #002752; */
+    font-weight: 700;
+    text-decoration: rgb(11, 46, 19);
+    text-decoration-color: rgb(11, 46, 19);
+    text-decoration-line: none;
+    text-decoration-style: solid;
+    text-decoration-thickness: auto;
+  }
+.bk.alert a:hover {
+    color: rgb(11, 46, 19);
+    font-weight: 700;
+    text-decoration: underline;
+}
+.bk.alert-danger {
+    color: #721c24;
+    background-color: #f8d7da;
+    border-color: #f5c6cb;
+}
+.bk.alert-danger hr {
+    border-top-color: #f1b0b7;
+}
+</style>
+"""
+
+
+class ConversionError(Exception):
+    """Raised if the app cannot be converted"""
+
 
 def _convert_project(app: str = "source/app.py", dest_path: str = "build", requirements="auto"):
     """Helper function"""
-    Path(dest_path).mkdir(parents=True, exist_ok=True)
-    convert_app(app=Path(app), dest_path=Path(dest_path), requirements=requirements)  # type: ignore
+    build_dir = Path(dest_path)
+    build_dir.mkdir(parents=True, exist_ok=True)
+
+    temp_out = StringIO()
+    sys.stdout = temp_out
+    convert_app(
+        app=Path(app), # type: ignore
+        dest_path=Path(dest_path), # type: ignore
+        requirements=requirements,
+        prerender=True
+    )
+
+    if not ((build_dir / "app.html").exists() and (build_dir / "app.js").exists()):
+        temp_out = StringIO()
+        sys.stdout = temp_out
+        convert_app(
+            app=Path(app), # type: ignore
+            dest_path=Path(dest_path), # type: ignore
+            requirements=requirements,
+            prerender=False
+        )  # type: ignore
+
+    out = temp_out.getvalue()
+    sys.stdout = sys.__stdout__
+    if (build_dir / "app.html").exists() and (build_dir / "app.js").exists():
+        return
+
+    if out.startswith("Failed to convert"):
+        out = """\n\nFailed to convert. Make sure you are 1) marking items .servable() and 2) only
+importing packages available on the server."""
+
+    app_html_path = Path(dest_path) / "app.html"
+    app_html_path.parent.mkdir(parents=True, exist_ok=True)
+    app_html_path.write_text(
+        f"""
+{ALERT_STYLE}<div class="bk alert alert-danger">{out}</div>
+"""
+    )
+    raise ConversionError(out)
 
 
 class Source(param.Parameterized):
@@ -117,7 +193,7 @@ class Project(param.Parameterized):
             params["source"] = Source()
         super().__init__(**params)
 
-        self._tmpdir = tempfile.TemporaryDirectory() # pylint: disable=consider-using-with
+        self._tmpdir = tempfile.TemporaryDirectory()  # pylint: disable=consider-using-with
         self._tmppath = Path(self._tmpdir.name)
         self._save_hash = ""
         self._build_hash = ""
@@ -139,9 +215,9 @@ class Project(param.Parameterized):
         with set_directory(Path("source")):
             project.source = Source.read()
         shutil.copytree(Path(), Path(project._tmpdir.name), dirs_exist_ok=True)
-        project._save_hash=project._hash
+        project._save_hash = project._hash
         if Path("build").exists():
-            project._build_hash=project._hash
+            project._build_hash = project._hash
         return project
 
     @property
@@ -149,8 +225,8 @@ class Project(param.Parameterized):
         return hash(json.dumps(self.to_dict()))
 
     def _reset_tmpdir(self):
-        self._save_hash=""
-        self._build_hash=""
+        self._save_hash = ""
+        self._build_hash = ""
         for child in self._tmppath.iterdir():
             if child.is_file():
                 child.unlink()
@@ -159,11 +235,11 @@ class Project(param.Parameterized):
 
     def _save_to_tmpdir(self):
         _hash = self._hash
-        if self._save_hash==_hash:
+        if self._save_hash == _hash:
             return
 
         self._reset_tmpdir()
-        with set_directory(self._tmppath/"source"):
+        with set_directory(self._tmppath / "source"):
             self.source.save()
         self._save_hash = _hash
 
@@ -207,14 +283,14 @@ class Project(param.Parameterized):
             json.dump(obj=build_json, fp=file, indent=1)
 
     def _remove_tmpbuilddir(self):
-        self._build_hash=""
-        tmpbuildpath = self._tmppath/"build"
+        self._build_hash = ""
+        tmpbuildpath = self._tmppath / "build"
         if tmpbuildpath.exists():
             shutil.rmtree(tmpbuildpath)
 
     def _build_to_tmpdir(self, base_target):
         _hash = self._hash + hash(base_target)
-        if self._build_hash==_hash:
+        if self._build_hash == _hash:
             return
 
         self._remove_tmpbuilddir()
@@ -229,7 +305,6 @@ class Project(param.Parameterized):
                     kwargs=kwargs,
                 )
                 process.start()
-
                 with set_directory(Path("build")):
                     self.save_build_json(kwargs)
                 process.join()
@@ -253,8 +328,8 @@ class Project(param.Parameterized):
 
     def rebuild(self, base_target=""):
         """Forces as rebuild"""
-        self._save_hash=""
-        self._build_hash=""
+        self._save_hash = ""
+        self._build_hash = ""
         self.build(base_target=base_target)
 
     def to_dict(self):
@@ -314,17 +389,15 @@ class Project(param.Parameterized):
         self.source.readme = project.source.readme
         self.source.requirements = project.source.requirements
         self.source.thumbnail = project.source.thumbnail
-        self._save_hash=""
-        self._build_hash=""
+        self._save_hash = ""
+        self._build_hash = ""
         self._reset_tmpdir()
 
         # pylint: disable=protected-access
         with set_directory(self._tmppath):
             project._copy_from_tmpdir()
-        self._save_hash=project._save_hash
-        self._build_hash=project._build_hash
-
-
+        self._save_hash = project._save_hash
+        self._build_hash = project._build_hash
 
 
 class User(param.Parameterized):
@@ -577,11 +650,10 @@ class AzureBlobStorage(Storage):
             return self.web_url + key + "/" + str(file_path)
         return self.blob_url + container_name + "/" + key + "/" + str(file_path)
 
-
     def get_keys(self) -> List[str]:
         """Returns the app keys from the blob storage"""
         keys = []
-        for user in self.web_container_client.walk_blobs('', delimiter='/'):
+        for user in self.web_container_client.walk_blobs("", delimiter="/"):
             if isinstance(user, BlobPrefix):
                 for app in user:
                     if isinstance(app, BlobPrefix):
